@@ -1,8 +1,8 @@
 /*
  * This file is part of vncviewer.
  *
- * Copyright(C) 2005, 2006 Aaron Levinson.
- * Copyright(C) 2006, Detlef Schmicker
+ * Copyright(C) 2005-2007, Aaron Levinson
+ * Copyright(C) 2006-2007, Detlef Schmicker
  *
  *
  * This software is free software; you can redistribute it and/or
@@ -39,6 +39,48 @@
 #include <hildon-widgets/hildon-note.h>
 #include <hildon-widgets/hildon-get-password-dialog.h>
 #include <hildon-widgets/hildon-name-password-dialog.h>
+#include <hildon-widgets/hildon-banner.h>
+
+
+/*
+// !!!!!!!!!!!!!!!!! Fake code from Tomas Junnonen, nokia
+#include <X11/extensions/XTest.h>
+
+#define SELECT_KEY_LONG_PRESS 1000
+#define FINGER_PRESS_BUTTON 8
+
+guint timeout_id;
+
+static gboolean
+select_key_long_press(GdkWindow * window)
+{
+  //  GtkWidget *entry = GTK_WIDGET(data);
+  gint x=100, y=100, width, height;
+  g_message("Fake 1a");
+
+  //  gdk_window_get_origin(window, &x, &y);
+  //g_message("Fake 1b");
+  //gdk_drawable_get_size(window, &width, &height);
+  //g_message("Fake 1c");
+  //if (width && height)
+  {
+  g_message("Fake 1d");
+    XTestFakeMotionEvent(GDK_DISPLAY(), -1, 
+                         x + (width / 2), y + (height / 2), 0);
+  g_message("Fake 2");
+    XTestFakeButtonEvent(GDK_DISPLAY(), FINGER_PRESS_BUTTON, TRUE, 0);
+  g_message("Fake 3");
+    XTestFakeButtonEvent(GDK_DISPLAY(), FINGER_PRESS_BUTTON, FALSE, 0);
+  g_message("Fake 4");
+  }
+
+  timeout_id = 0;
+  return FALSE;
+}
+
+//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!111
+*/
+
 
 #include "rfb.h"
 
@@ -76,9 +118,14 @@ static void scrolledwin_size_allocate(GtkWidget *widget,
 
 static gboolean mainview_auto_connect(gpointer userdata);
 static gboolean make_connection(const gchar *display, gboolean shared,
-				VncViewerDepthType depth, MainView *mainview);
+				VncViewerDepthType depth,
+				VncViewerRotationType rotationType,
+				MainView *mainview);
 static gboolean app_key_snooper(GtkWidget *widget, GdkEventKey *keyevent,
 				HildonProgram *app);
+static void about_dialog_url_hook_function(GtkAboutDialog *about,
+					   const gchar *link,
+					   gpointer data);
 
 // Creates and initializes a main_view
 MainView* interface_main_view_new(AppData *data)
@@ -115,9 +162,16 @@ MainView* interface_main_view_new(AppData *data)
     result->default_opts.m_nDepth =
       (VncViewerDepthType) gconf_client_get_int(result->gc_client,
 						MAEMO_VNCVIEWER_GCONF_DEPTH, 0);
+    // if not found, 0 is also the default
+    result->default_opts.m_nRotationType =
+      (VncViewerDepthType) gconf_client_get_int(result->gc_client,
+						MAEMO_VNCVIEWER_GCONF_ROTATION, 0);
     // i.e. if not found, use the default
     if (!result->default_opts.m_nDepth)
       result->default_opts.m_nDepth = VNC_VIEWER_DEPTH_DEFAULT;
+    else if (result->default_opts.m_nDepth == VNC_VIEWER_DEPTH_24BIT)
+      // 24 bit is no longer supported
+      result->default_opts.m_nDepth = VNC_VIEWER_DEPTH_16BIT;
 
     // using ! to allow a missing gconf value to work properly
     // avoids needing to examine a GError
@@ -156,6 +210,9 @@ MainView* interface_main_view_new(AppData *data)
 	result->m_bHaveHomeKeyLock = TRUE;
     }
 
+    gtk_about_dialog_set_url_hook(about_dialog_url_hook_function,
+				  data->osso, 0);
+
     // Store handle to app's data to view's data
     result->data = data;
     // Store handle to HildonWindow to app's data
@@ -234,6 +291,8 @@ void interface_main_view_destroy(MainView *mainview)
 			mainview->default_opts.m_bShared, 0);
   gconf_client_set_int(mainview->gc_client, MAEMO_VNCVIEWER_GCONF_DEPTH,
 		       (gint) mainview->default_opts.m_nDepth, 0);
+  gconf_client_set_int(mainview->gc_client, MAEMO_VNCVIEWER_GCONF_ROTATION,
+		       (gint) mainview->default_opts.m_nRotationType, 0);
 
   // using ! as per comments in interface_main_view_new()
   gconf_client_set_bool(mainview->gc_client,
@@ -284,8 +343,7 @@ void options_button_clicked(GtkButton *button,
   GtkWidget *parent_dialog = GTK_WIDGET(data);
   GtkWidget *dialog, *shared_button;
   GtkWidget *vbox, *frame;
-  GtkToggleButton *default_radio, *eight_bit_radio, *sixteen_bit_radio,
-    *twenty_four_bit_radio;
+  GtkToggleButton *default_radio, *eight_bit_radio, *sixteen_bit_radio;
   GtkToggleButton *toggle = 0;
 
   dialog = gtk_dialog_new_with_buttons("VNC Viewer: Options",
@@ -325,13 +383,10 @@ void options_button_clicked(GtkButton *button,
     GTK_TOGGLE_BUTTON(gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(default_radio), "_8-bit"));
   sixteen_bit_radio =
     GTK_TOGGLE_BUTTON(gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(eight_bit_radio), "_16-bit"));
-  twenty_four_bit_radio =
-    GTK_TOGGLE_BUTTON(gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(sixteen_bit_radio), "_24-bit"));
 
   gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(default_radio));
   gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(eight_bit_radio));
   gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(sixteen_bit_radio));
-  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(twenty_four_bit_radio));
 
   switch (pOpts->m_nDepth)
   {
@@ -347,8 +402,47 @@ void options_button_clicked(GtkButton *button,
     toggle = sixteen_bit_radio;
     break;
   
-  case VNC_VIEWER_DEPTH_24BIT:
-    toggle = twenty_four_bit_radio;
+  default:
+    break;
+  }
+
+  gtk_toggle_button_set_active(toggle, TRUE);
+
+  // rotation
+  GtkWidget *rotation_frame = gtk_frame_new("Rotation type");
+  gtk_frame_set_shadow_type(GTK_FRAME(rotation_frame), GTK_SHADOW_ETCHED_IN);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), rotation_frame,
+		     TRUE, TRUE, 5);
+
+  GtkWidget *rotation_vbox = gtk_vbox_new(TRUE, 2);
+  gtk_container_add(GTK_CONTAINER(rotation_frame), rotation_vbox);
+
+  GtkToggleButton *auto_rotate_radio =
+    GTK_TOGGLE_BUTTON(gtk_radio_button_new_with_mnemonic(0, "_Automatically rotate"));
+  GtkToggleButton *always_rotate_radio =
+    GTK_TOGGLE_BUTTON(gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(auto_rotate_radio), "Always _rotate"));
+  GtkToggleButton *never_rotate_radio =
+    GTK_TOGGLE_BUTTON(gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(always_rotate_radio), "_Never rotate"));
+
+  gtk_container_add(GTK_CONTAINER(rotation_vbox),
+		    GTK_WIDGET(auto_rotate_radio));
+  gtk_container_add(GTK_CONTAINER(rotation_vbox),
+		    GTK_WIDGET(always_rotate_radio));
+  gtk_container_add(GTK_CONTAINER(rotation_vbox),
+		    GTK_WIDGET(never_rotate_radio));
+
+  switch (pOpts->m_nRotationType)
+  {
+  case VNC_VIEWER_ROTATE_AUTO:
+    toggle = auto_rotate_radio;
+    break;
+  
+  case VNC_VIEWER_ROTATE_ALWAYS:
+    toggle = always_rotate_radio;
+    break;
+  
+  case VNC_VIEWER_ROTATE_NEVER:
+    toggle = never_rotate_radio;
     break;
   
   default:
@@ -370,8 +464,13 @@ void options_button_clicked(GtkButton *button,
       pOpts->m_nDepth = VNC_VIEWER_DEPTH_8BIT;
     else if (gtk_toggle_button_get_active(sixteen_bit_radio))
       pOpts->m_nDepth = VNC_VIEWER_DEPTH_16BIT;
-    else if (gtk_toggle_button_get_active(twenty_four_bit_radio))
-      pOpts->m_nDepth = VNC_VIEWER_DEPTH_24BIT;
+
+    if (gtk_toggle_button_get_active(auto_rotate_radio))
+      pOpts->m_nRotationType = VNC_VIEWER_ROTATE_AUTO;
+    else if (gtk_toggle_button_get_active(always_rotate_radio))
+      pOpts->m_nRotationType = VNC_VIEWER_ROTATE_ALWAYS;
+    else if (gtk_toggle_button_get_active(never_rotate_radio))
+      pOpts->m_nRotationType = VNC_VIEWER_ROTATE_NEVER;
   }
 
   gtk_widget_destroy(dialog);
@@ -458,7 +557,9 @@ static void vnc_connection_died(VncViewer *vnc, gpointer data) {
     {
       if (make_connection((const gchar *) arg_hostname,
 			  mainview->default_opts.m_bShared,
-			  mainview->default_opts.m_nDepth, mainview) ||
+			  mainview->default_opts.m_nDepth,
+			  mainview->default_opts.m_nRotationType,
+			  mainview) ||
 	  !arg_lockdown)
 	break;
     }
@@ -466,7 +567,9 @@ static void vnc_connection_died(VncViewer *vnc, gpointer data) {
 }
 
 static gboolean make_connection(const gchar *display, gboolean shared,
-				VncViewerDepthType depth, MainView *mainview)
+				VncViewerDepthType depth,
+				VncViewerRotationType rotationType,
+				MainView *mainview)
 {
   gint fd;
   gint32 res, ret;
@@ -477,7 +580,19 @@ static gboolean make_connection(const gchar *display, gboolean shared,
   GtkWidget *msgDialog;
   GtkAdjustment *hadj, *vadj;
 
+  // Display an information, as connecting may take long, if the wlan is not yet connected
+  GtkWidget * banner =hildon_banner_show_animation(GTK_WIDGET(mainview->data->main_view),
+				 "Connecting...","Connecting...");
+  // Wait a while for the xwindows system having handled the banner window
+  int tmpi=0;
+  while (tmpi++<800)
+    gtk_main_iteration_do(FALSE);
+
   fd = connectToServer(display);
+
+  //connection ready (or not) connecting banner can disapear
+  gtk_widget_destroy(GTK_WIDGET(banner));
+
   if (fd < 0 || (res = doVersionHandshake(fd)) < 0)
   {
     if (arg_lockdown)
@@ -619,31 +734,81 @@ static gboolean make_connection(const gchar *display, gboolean shared,
   // create VncViewer widget
   mainview->vnc = VNC_VIEWER(vnc_viewer_new(fd, shared, depth));
   mainview->vnc->mainview = (void *) mainview;
+
+  switch (rotationType)
+  {
+  case VNC_VIEWER_ROTATE_AUTO:
+    if (mainview->vnc->height > mainview->vnc->width)
+      mainview->vnc->m_bRotateDisplay = TRUE;
+    else
+      mainview->vnc->m_bRotateDisplay = FALSE;
+    break;
+
+  case VNC_VIEWER_ROTATE_ALWAYS:
+    mainview->vnc->m_bRotateDisplay = TRUE;
+    break;
+
+  case VNC_VIEWER_ROTATE_NEVER:
+    mainview->vnc->m_bRotateDisplay = FALSE;
+    break;
+
+  default:
+    // shouldn't get here
+    mainview->vnc->m_bRotateDisplay = FALSE;
+    break;
+  }
+
   vnc_viewer_set_cursor(mainview->vnc, VNC_VIEWER_CURSOR_DOT);
   g_signal_connect(G_OBJECT(mainview->vnc), "connection_died",
 		   G_CALLBACK(vnc_connection_died), mainview);
 
   // make the size of the VncViewer widget the size returned by the server
-  gtk_widget_set_size_request(GTK_WIDGET(mainview->vnc),
-			      mainview->vnc->width,
-			      mainview->vnc->height);
+  if (!mainview->vnc->m_bRotateDisplay)
+  {
+    gtk_widget_set_size_request(GTK_WIDGET(mainview->vnc),
+				mainview->vnc->width,
+				mainview->vnc->height);
+  }
+  else
+  {
+    gtk_widget_set_size_request(GTK_WIDGET(mainview->vnc),
+				mainview->vnc->height,
+				mainview->vnc->width);
+  }
 
   // set up adjustments for the scrolled window
   hadj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(mainview->scrolledwin));
-  hadj->lower = 0;
-  hadj->upper = mainview->vnc->width;
+  if (!mainview->vnc->m_bRotateDisplay)
+  {
+    hadj->lower = 0;
+    hadj->upper = mainview->vnc->width;
+  }
+  else
+  {
+    hadj->lower = mainview->vnc->height;
+    hadj->upper = 0;
+  }
   hadj->value = 0;
   hadj->step_increment = 20;
-  hadj->page_increment = GTK_WIDGET(mainview->scrolledwin)->allocation.width;
+  if (!mainview->vnc->m_bRotateDisplay)
+    hadj->page_increment = GTK_WIDGET(mainview->scrolledwin)->allocation.width;
+  else
+    hadj->page_increment = GTK_WIDGET(mainview->scrolledwin)->allocation.height;
   hadj->page_size = hadj->page_increment;
   gtk_adjustment_changed(hadj);
   
   vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(mainview->scrolledwin));
   vadj->lower = 0;
-  vadj->upper = mainview->vnc->height;
+  if (!mainview->vnc->m_bRotateDisplay)
+    vadj->upper = mainview->vnc->height;
+  else
+    vadj->upper = mainview->vnc->width;
   vadj->value = 0;
   vadj->step_increment = 20;
-  vadj->page_increment = GTK_WIDGET(mainview->scrolledwin)->allocation.height;
+  if (!mainview->vnc->m_bRotateDisplay)
+    vadj->page_increment = GTK_WIDGET(mainview->scrolledwin)->allocation.height;
+  else
+    vadj->page_increment = GTK_WIDGET(mainview->scrolledwin)->allocation.width;
   vadj->page_size = vadj->page_increment;
   gtk_adjustment_changed(vadj);
   
@@ -691,6 +856,7 @@ static void callback_connect(GtkBin *action, gpointer data)
   options_settings opts;
   opts.m_bShared = mainview->default_opts.m_bShared;
   opts.m_nDepth = mainview->default_opts.m_nDepth;
+  opts.m_nRotationType = mainview->default_opts.m_nRotationType;
   // the "options" data is used by the Options dialog, it allows the opts
   // struct to be shared between the two dialogs
   g_object_set_data(G_OBJECT(dialog), "options", &opts);
@@ -758,11 +924,13 @@ static void callback_connect(GtkBin *action, gpointer data)
     if (mainview->vnc)
       vnc_connection_closed(mainview);
 
-    make_connection(display, opts.m_bShared, opts.m_nDepth, mainview);
+    make_connection(display, opts.m_bShared, opts.m_nDepth,
+		    opts.m_nRotationType, mainview);
 
     // preserve settings, even if the connection failed
     mainview->default_opts.m_bShared = opts.m_bShared;
     mainview->default_opts.m_nDepth = opts.m_nDepth;
+    mainview->default_opts.m_nRotationType = opts.m_nRotationType;
 
     nActive = gtk_combo_box_get_active(host_entry_combo);
     if (nActive != -1)
@@ -989,16 +1157,15 @@ void callback_about(GtkBin *action, gpointer data)
   const gchar *authors[3] = { "Aaron Levinson <alevinsn@aracnet.com>",
 			      "Detlef Schmicker <d.schmicker@physik.de>", 0 };
   //const gchar *documenters[] = { 0 };
-  //gtk_about_dialog_set_url_hook(0, 0, 0);
   gtk_show_about_dialog(GTK_WINDOW(mainview->data->main_view),
 			"name",_("VNC Viewer"),
 			"version", VERSION,
-			"copyright", "Copyright(C) 2006 Aaron Levinson, Detlef Schmicker",
+			"copyright", "Copyright(C) 2006-2007 Aaron Levinson, Detlef Schmicker",
 			//"comments", _("A VNC viewer for maemo"),
 			"authors", authors,
 			//"documenters", documenters,
 			//"translator-credits", 0,
-			"website", "garage.vncviewer.maemo.org",
+			"website", "http://vncviewer.garage.maemo.org/",
 			"website-label", _("Home page"),
 			"logo-icon-name", "vncviewer",
 			0);
@@ -1085,6 +1252,11 @@ void callback_keys_menu_deactivate(GtkMenuShell *menushell, gpointer userdata)
 				    FALSE);
 }
 
+gboolean was_scrolling = FALSE;
+// if scrolling is changing from local to remote window, 
+// let the user have a better chance to stop scrolling
+gint wait_scrolling = 3;
+
 gboolean callback_keypress(GtkWidget *widget, GdkEventKey *event,
 			   gpointer userdata)
 {
@@ -1112,6 +1284,7 @@ gboolean callback_keypress(GtkWidget *widget, GdkEventKey *event,
   case GDK_Down:
   case GDK_Left:
   case GDK_Right:
+    was_scrolling = FALSE;
     if (mainview->vnc &&
 	(gtk_widget_is_focus(GTK_WIDGET(mainview->vnc)) == TRUE))
     {
@@ -1128,12 +1301,35 @@ gboolean callback_keypress(GtkWidget *widget, GdkEventKey *event,
 
 	  if (event->keyval == GDK_Up)
 	  {
+	    //Detlef Testing
+	    if (vadj->value==0) 
+	    {
+	      if (wait_scrolling<=0) 
+	      {
+		was_scrolling=TRUE;
+		vnc_viewer_key_press_event(GTK_WIDGET(mainview->vnc), event);
+		vnc_viewer_key_release_event(GTK_WIDGET(mainview->vnc), event);
+	      }
+	      else
+		wait_scrolling--;
+	    }
 	    vadj->value = vadj->value - vadj->step_increment;
 	    if (vadj->value < vadj->lower)
 	      vadj->value = vadj->lower;
 	  }
 	  else
 	  {
+	    if (vadj->value+vadj->page_size>=vadj->upper) 
+	    {
+	      if (wait_scrolling<=0) 
+	      {
+		was_scrolling=TRUE;
+		vnc_viewer_key_press_event(GTK_WIDGET(mainview->vnc), event);
+		vnc_viewer_key_release_event(GTK_WIDGET(mainview->vnc), event);
+	      }
+	      else
+		wait_scrolling--;
+	    }
 	    vadj->value = vadj->value + vadj->step_increment;
 	    if (vadj->value > vadj->upper - vadj->page_size)
 	      vadj->value = vadj->upper - vadj->page_size;
@@ -1146,12 +1342,34 @@ gboolean callback_keypress(GtkWidget *widget, GdkEventKey *event,
 	  GtkAdjustment *hadj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(mainview->scrolledwin));
 	  if (event->keyval == GDK_Left)
 	  {
+	    if (hadj->value==0) 
+	    {
+	      if (wait_scrolling<=0) 
+	      {
+		was_scrolling=TRUE;
+		vnc_viewer_key_press_event(GTK_WIDGET(mainview->vnc), event);
+		vnc_viewer_key_release_event(GTK_WIDGET(mainview->vnc), event);
+	      }
+	      else
+		wait_scrolling--;
+	    }
 	    hadj->value = hadj->value - hadj->step_increment;
 	    if (hadj->value < hadj->lower)
 	      hadj->value = hadj->lower;
 	  }
 	  else
 	  {
+	    if (hadj->value+hadj->page_size>=hadj->upper) 
+	    {
+	      if (wait_scrolling<=0) 
+	      {
+		was_scrolling=TRUE;
+		vnc_viewer_key_press_event(GTK_WIDGET(mainview->vnc), event);
+		vnc_viewer_key_release_event(GTK_WIDGET(mainview->vnc), event);
+	      }
+	      else
+		wait_scrolling--;
+	    }
 	    hadj->value = hadj->value + hadj->step_increment;
 	    if (hadj->value > hadj->upper - hadj->page_size)
 	      hadj->value = hadj->upper - hadj->page_size;
@@ -1173,6 +1391,18 @@ gboolean callback_keypress(GtkWidget *widget, GdkEventKey *event,
     break;
   }
 
+  /*
+  if (event->keyval == HILDON_HARDKEY_SELECT){
+    g_message("HILDON_HARDKEY press");
+    if (timeout_id == 0)
+      timeout_id = g_timeout_add(SELECT_KEY_LONG_PRESS,
+				 select_key_long_press,
+				 GTK_WIDGET(userdata)->window);
+
+    return FALSE;
+  }
+  */
+
   return FALSE;
 }
 
@@ -1180,7 +1410,8 @@ gboolean callback_keyrelease(GtkWidget *widget, GdkEventKey *event,
 			     gpointer userdata)
 {
   MainView *mainview = (MainView *) userdata;
-  
+  wait_scrolling=3;
+
   if (arg_lockdown)
     return FALSE;
 
@@ -1197,11 +1428,14 @@ gboolean callback_keyrelease(GtkWidget *widget, GdkEventKey *event,
       //event->window = (GTK_WIDGET(mainview->vnc))->window;
       //gtk_widget_event(GTK_WIDGET(mainview->vnc), (GdkEvent *) event);
       //gtk_main_do_event((GdkEvent *) event);
-      vnc_viewer_key_press_event(GTK_WIDGET(mainview->vnc), event);
-      //event->type = GDK_KEY_RELEASE;
-      //gtk_widget_event(GTK_WIDGET(mainview->vnc), (GdkEvent *) event);
-      //gtk_main_do_event((GdkEvent *) event);
-      vnc_viewer_key_release_event(GTK_WIDGET(mainview->vnc), event);
+      if (!was_scrolling) 
+      {
+	vnc_viewer_key_press_event(GTK_WIDGET(mainview->vnc), event);
+	//event->type = GDK_KEY_RELEASE;
+	//gtk_widget_event(GTK_WIDGET(mainview->vnc), (GdkEvent *) event);
+	//gtk_main_do_event((GdkEvent *) event);
+	vnc_viewer_key_release_event(GTK_WIDGET(mainview->vnc), event);
+      }
 
       mainview->directionKeyPress = 0;
       return TRUE;
@@ -1211,6 +1445,19 @@ gboolean callback_keyrelease(GtkWidget *widget, GdkEventKey *event,
   mainview->directionKeyPress = 0;
   mainview->bMultiplePresses = 0;
 
+  /*
+  if (event->keyval == HILDON_HARDKEY_SELECT){
+    g_message("HILDON_HARDKEY release");
+    if (timeout_id)
+      {
+	g_source_remove(timeout_id);
+	timeout_id = 0;
+      }
+
+    return FALSE;
+  }
+  */
+
   return FALSE;
 }
 
@@ -1227,8 +1474,11 @@ void scrolledwin_size_allocate(GtkWidget *widget,
 				 &hpolicy, 0);
 
   if ((hpolicy == GTK_POLICY_AUTOMATIC) &&
-      (allocation->width >= mainview->vnc->width) &&
-      (allocation->height >= mainview->vnc->height))
+      (!mainview->vnc->m_bRotateDisplay ?
+      ((allocation->width >= mainview->vnc->width) &&
+       (allocation->height >= mainview->vnc->height)) :
+      ((allocation->width >= mainview->vnc->height) &&
+       (allocation->height >= mainview->vnc->width))))
   {
     // then the server desktop size will fit in the window--cause the scroll
     // bars to be turned off
@@ -1285,7 +1535,9 @@ static gboolean mainview_auto_connect(gpointer userdata)
       {
 	if (make_connection((const gchar *) arg_hostname,
 			    mainview->default_opts.m_bShared,
-			    mainview->default_opts.m_nDepth, mainview))
+			    mainview->default_opts.m_nDepth,
+			    mainview->default_opts.m_nRotationType,
+			    mainview))
 	{
 	  // move it to the beginning of m_hostList if the node already exists
 	  GSList *hostNode = g_slist_find_custom(mainview->m_hostList,
@@ -1348,6 +1600,7 @@ static void create_menu_helper(MainView *mainview)
   GtkWidget *toolbar_item;
   GtkWidget *connect_item;
   //GtkWidget *take_screenshot_item;
+  GtkWidget *fullscreen_item;
   GtkWidget *normal_toolbar_item;
   GtkWidget *full_toolbar_item;
   GtkWidget *show_scrollbars_item;
@@ -1366,11 +1619,8 @@ static void create_menu_helper(MainView *mainview)
   gtk_widget_set_sensitive(mainview->disconnect_item, FALSE);
 
   mainview->view_item = gtk_menu_item_new_with_label(_("View"));
-  mainview->fullscreen_item =
-    gtk_check_menu_item_new_with_label(_("Full screen"));
+  fullscreen_item = gtk_menu_item_new_with_label(_("Full screen"));
   toolbar_item = gtk_menu_item_new_with_label(_("Show toolbar"));
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainview->fullscreen_item),
-				 mainview->fullscreen);
 
   normal_toolbar_item =
     gtk_check_menu_item_new_with_label(_("Normal screen"));
@@ -1393,7 +1643,7 @@ static void create_menu_helper(MainView *mainview)
   // Add menu items to right menus
   gtk_menu_append(file_menu, connect_item);
   gtk_menu_append(file_menu, mainview->disconnect_item);
-  gtk_menu_append(view_menu, mainview->fullscreen_item);
+  gtk_menu_append(view_menu, fullscreen_item);
   gtk_menu_append(view_menu, toolbar_item);
   gtk_menu_append(toolbar_menu, normal_toolbar_item);
   gtk_menu_append(toolbar_menu, full_toolbar_item);
@@ -1411,7 +1661,7 @@ static void create_menu_helper(MainView *mainview)
 		   G_CALLBACK(callback_connect), mainview);
   g_signal_connect(G_OBJECT(mainview->disconnect_item), "activate", 
 		   G_CALLBACK(callback_disconnect), mainview);
-  g_signal_connect(G_OBJECT(mainview->fullscreen_item), "toggled", 
+  g_signal_connect(G_OBJECT(fullscreen_item), "activate", 
 		   G_CALLBACK(callback_fullscreen), mainview);
   g_signal_connect(G_OBJECT(normal_toolbar_item), "toggled", 
 		   G_CALLBACK(callback_normal_toggled), mainview);
@@ -1459,6 +1709,7 @@ void create_menu(void *mainvw)
 static void create_toolbar(MainView *mainview)
 {
     GtkToolItem *separator_tb1;
+    GtkToolItem *fullscreen_tb;
     GtkToolItem *separator_tb2;
     GtkToolItem *quit_tb;
 
@@ -1511,10 +1762,7 @@ static void create_toolbar(MainView *mainview)
     separator_tb1 = gtk_separator_tool_item_new();
     // the following stock item only in GTK 2.8
     //fullscreen_tb = gtk_tool_button_new_from_stock(GTK_STOCK_FULLSCREEN);
-    mainview->fullscreen_tb =
-      gtk_toggle_tool_button_new_from_stock(GTK_STOCK_ZOOM_FIT);
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(mainview->fullscreen_tb),
-				      mainview->fullscreen);
+    fullscreen_tb = gtk_tool_button_new_from_stock(GTK_STOCK_ZOOM_FIT);
     mainview->im_tb = gtk_tool_button_new_from_stock(GTK_STOCK_EDIT);
     gtk_widget_set_sensitive(GTK_WIDGET(mainview->im_tb), FALSE);
     mainview->keys_tb = gtk_toggle_tool_button_new();
@@ -1531,7 +1779,7 @@ static void create_toolbar(MainView *mainview)
     gtk_toolbar_insert(GTK_TOOLBAR(mainview->toolbar), mainview->connect_tb, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(mainview->toolbar), mainview->disconnect_tb, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(mainview->toolbar), separator_tb1, -1);
-    gtk_toolbar_insert(GTK_TOOLBAR(mainview->toolbar), mainview->fullscreen_tb, -1);
+    gtk_toolbar_insert(GTK_TOOLBAR(mainview->toolbar), fullscreen_tb, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(mainview->toolbar), mainview->im_tb, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(mainview->toolbar), mainview->keys_tb, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(mainview->toolbar), separator_tb2, -1);
@@ -1542,7 +1790,7 @@ static void create_toolbar(MainView *mainview)
                      G_CALLBACK(callback_connect), mainview);
     g_signal_connect(G_OBJECT(mainview->disconnect_tb), "clicked",
                      G_CALLBACK(callback_disconnect), mainview);
-    g_signal_connect(G_OBJECT(mainview->fullscreen_tb), "toggled",
+    g_signal_connect(G_OBJECT(fullscreen_tb), "clicked",
                      G_CALLBACK(callback_fullscreen), mainview);
     g_signal_connect(G_OBJECT(mainview->im_tb), "clicked",
                      G_CALLBACK(callback_im), mainview);
@@ -1757,3 +2005,15 @@ static gboolean app_key_snooper(GtkWidget *widget, GdkEventKey *keyevent,
 
   return FALSE;
 }
+
+static void about_dialog_url_hook_function(GtkAboutDialog *about,
+					   const gchar *link,
+					   gpointer data)
+{
+  if (!data)
+    return;
+
+  osso_context_t *osso = (osso_context_t *) data;
+  osso_rpc_run_with_defaults(osso, "osso_browser", "open_new_window", 0,
+			     DBUS_TYPE_STRING, link, DBUS_TYPE_INVALID);
+}
