@@ -33,10 +33,27 @@
 #include <hildon-widgets/hildon-get-password-dialog.h>
 #include <hildon-widgets/hildon-note.h>
 #include "vnc-viewer.h"
+#include "interface.h"
 
 #include <string.h>
 #include <unistd.h>
 
+// Timer to test the performance, Macro perf(A) replaced by nothing 
+// if performance measurement not wanted
+//#define perf
+
+#ifdef perf
+GTimer * Tperf=NULL;
+gdouble T1=0,T2=0,T3=0;
+guint Tcount=0;
+int frames=0;
+#endif
+
+extern guint32 ReadBytes;
+
+//gdkvisual16 is the format of the display if it supports 16 bit depth
+GdkVisual * gdkvisual16=NULL;
+
 extern void create_menu(void *mainvw);
 extern void disable_home_key_lock(void *mainvw);
 
@@ -315,6 +332,8 @@ static void vnc_viewer_queue_event(VncViewer *vnc, rfbClientToServerMsg *msg)
 				 vnc);
 }
 
+
+GdkRgbCmap *cmap;
 /* fd is a connection to the VNC server just after the connection has been
  * authenticated.  Shared says whether we allow other concurrent connections,
  * and bpp is the desired bpp
@@ -324,7 +343,7 @@ void vnc_viewer_configure(VncViewer *vnc, gint fd, gboolean shared,
 {
   guint32 encodings[] =
   { 
-    //    rfbEncodingHextile,
+    rfbEncodingHextile,
     rfbEncodingCoRRE,
     rfbEncodingRRE,
     rfbEncodingCopyRect, rfbEncodingRaw
@@ -364,13 +383,40 @@ void vnc_viewer_configure(VncViewer *vnc, gint fd, gboolean shared,
   vnc->pixel_format.bigEndian = TRUE;
 #endif
 
+
+
+  //Checking the supported visuals
+  GList *visuals=gdk_list_visuals();
+  g_message("Supported visuals:");
+  visuals=g_list_first(visuals);
+  while (visuals)
+  {
+    g_message("-");
+    g_message(
+	      "visual depth %d, redshift %d, greenshift %d, blueshift %d, type %d TrueColor is %d",
+	      ((GdkVisual*)visuals->data)->depth,
+	      ((GdkVisual*)visuals->data)->red_shift,
+	      ((GdkVisual*)visuals->data)->green_shift,
+	      ((GdkVisual*)visuals->data)->blue_shift,
+	      ((GdkVisual*)visuals->data)->type,
+	      GDK_VISUAL_TRUE_COLOR
+	      );
+    if ((((GdkVisual*)visuals->data)->depth)==16)
+      gdkvisual16=visuals->data;
+    visuals=g_list_next(visuals);
+  }
+  //  if this is not fullfilled, the 16 bit format must be changed in the following switch
+  g_assert(gdkvisual16!=NULL);
+  g_assert(gdkvisual16->red_shift==11);
+  g_assert(gdkvisual16->green_shift==5);
+  g_assert(gdkvisual16->blue_shift==0);
+  g_assert(gdkvisual16->type==GDK_VISUAL_TRUE_COLOR);
+  
+
   switch (depth)
   {
-  case VNC_VIEWER_DEPTH_DEFAULT:
-    break;
-
   case VNC_VIEWER_DEPTH_8BIT:
-    if (vnc->pixel_format.depth > 8)
+    //    if (vnc->pixel_format.depth > 8)
     {
       // only change depth down
       vnc->pixel_format.bitsPerPixel = 8;
@@ -382,11 +428,35 @@ void vnc_viewer_configure(VncViewer *vnc, gint fd, gboolean shared,
       vnc->pixel_format.redShift = 0;
       vnc->pixel_format.greenShift = 3;
       vnc->pixel_format.blueShift = 6;
+
+      guint32 colors[256];
+      guint32 r,g,b,ii=0;
+      for (b=0;b<=vnc->pixel_format.blueMax;b++)
+      {
+	for (g=0;g<=vnc->pixel_format.greenMax;g++)
+	{
+	  for (r=0;r<=vnc->pixel_format.redMax;r++)
+	  {
+	    guint32 tmp=
+	      (((b<<8) |0xFF)/(vnc->pixel_format.blueMax+1)) |
+	      (((g<<16) |0xFFFF)/(vnc->pixel_format.greenMax+1)&0xFF00) |
+	      (((r<<24) |0xFFFFFF)/(vnc->pixel_format.redMax+1)&0xFF0000) ;
+	    //g_message("c %d - %8X",ii,tmp);
+	    colors[ii++]=tmp;
+	  }
+	  cmap=gdk_rgb_cmap_new(colors,256);
+	}
+      }
     }
     break;
 
+    // The N770 and N800 display has only 16 bit depth. Therefore we force
+    // this to be the highest depth
+    // Even the pixel format must match the display for hextile encoding working
+  case VNC_VIEWER_DEPTH_DEFAULT:
   case VNC_VIEWER_DEPTH_16BIT:
-    if (vnc->pixel_format.depth > 16)
+  case VNC_VIEWER_DEPTH_24BIT:
+    //if (vnc->pixel_format.depth > 16)
     {
       // only change depth down
       vnc->pixel_format.bitsPerPixel = 16;
@@ -401,6 +471,7 @@ void vnc_viewer_configure(VncViewer *vnc, gint fd, gboolean shared,
     }
     break;
 
+    /*
   case VNC_VIEWER_DEPTH_24BIT:
     // we want to force this format
     // if (vnc->pixel_format.depth > 24)
@@ -416,8 +487,11 @@ void vnc_viewer_configure(VncViewer *vnc, gint fd, gboolean shared,
       vnc->pixel_format.blueShift = 0;
     }
     break;
+    */
   }
 
+
+
   vnc_viewer_queue_event(vnc, newSetPixelFormatMsg(&(vnc->pixel_format)));
 
   /*
@@ -516,27 +590,44 @@ static gboolean vnc_viewer_send_draw_request(VncViewer *vnc)
   if (vnc->fd < 0)
     return FALSE;
 
-  vnc_viewer_queue_event(vnc, newFramebufferUpdateRequestMsg(TRUE, 0, 0,
-							     vnc->width,
-							     vnc->height));
+  //If the window can not be seen by the user, no update request is sent
+  //But TRUE is returned for the function called again after timeout.
+  if (!gtk_window_is_active(GTK_WINDOW(((MainView*)vnc->mainview)->data->main_view)))
+    return TRUE;    
+
+  vnc_viewer_queue_event(vnc,
+			 newFramebufferUpdateRequestMsg(TRUE, 0, 0, vnc->width, vnc->height));
   return FALSE;
 }
 
 static void vnc_viewer_framebuffer_update(VncViewer *vnc,
 					  rfbServerToClientMsg *msg)
 {
-  guint32 rect, srect, tile, i, j;
+  GdkImage * gdkimage;
+  guint32 rect, srect, tile, i, j, k, bgRGB;
   guchar *buffer;
+  guchar *buffer8;
   rfbPixelFormat *fmt = &(vnc->pixel_format);
-  guint8 bpp = fmt->bitsPerPixel, tmpR, tmpG, tmpB, bgR, bgG, bgB;
+  guint8 bpp = fmt->bitsPerPixel, tmpR, tmpG, tmpB;
   gdouble redScale = 255.0 / fmt->redMax, greenScale = 255.0 / fmt->greenMax,
     blueScale = 255.0 / fmt->blueMax;
-
+//  g_message("framebuffer update");
   for (rect = 0; rect < msg->fu.nRects; rect++)
   {
+#ifdef perf
+    if (Tperf==NULL) Tperf=g_timer_new();
+    T3+=g_timer_elapsed(Tperf,NULL);
+    g_timer_start(Tperf);
+#endif
+
     rfbRectangleDescription *desc =
       getNextRectangle(vnc->fd, vnc->pixel_format.bitsPerPixel / 8,
 		       !vnc->m_bGotFirstUpdate);
+#ifdef perf
+    T1+=g_timer_elapsed(Tperf,NULL);
+#endif
+    gdkimage=NULL;
+    buffer8=NULL;
     if (!vnc->m_bGotFirstUpdate)
       vnc->m_bGotFirstUpdate = TRUE;
 
@@ -556,6 +647,7 @@ static void vnc_viewer_framebuffer_update(VncViewer *vnc,
     switch (desc->header.encoding)
     {
     case rfbEncodingRaw:
+      //g_message("raw encoding");
       buffer = g_malloc(totPixels * 3);
       for (i = 0; i < totPixels; i++)
       {
@@ -578,26 +670,61 @@ static void vnc_viewer_framebuffer_update(VncViewer *vnc,
 	  g_assert_not_reached();
 	}
 
-	buffer[3*i]   = (pixel >> fmt->redShift & fmt->redMax) * redScale;
-	buffer[3*i+1] = (pixel >> fmt->greenShift & fmt->greenMax) *greenScale;
-	buffer[3*i+2] = (pixel >> fmt->blueShift & fmt->blueMax) * blueScale;
+	if (!vnc->m_bRotateDisplay)
+	{
+	  buffer[3*i]   = (pixel >> fmt->redShift & fmt->redMax) * redScale;
+	  buffer[3*i+1] = (pixel >> fmt->greenShift & fmt->greenMax) *greenScale;
+	  buffer[3*i+2] = (pixel >> fmt->blueShift & fmt->blueMax) * blueScale;
+	}
+	else
+	{
+	  j = rHeight * ((i % rWidth) + 1) - ((i / rWidth) + 1);
+	  buffer[3*j]   = (pixel >> fmt->redShift & fmt->redMax) * redScale;
+	  buffer[3*j+1] = (pixel >> fmt->greenShift & fmt->greenMax) *greenScale;
+	  buffer[3*j+2] = (pixel >> fmt->blueShift & fmt->blueMax) * blueScale;
+	}
+      }
+	
+      if (!vnc->m_bRotateDisplay)
+      {
+	gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
+			   desc->header.r.x, desc->header.r.y,
+			   rWidth, rHeight,
+			   GDK_RGB_DITHER_NORMAL, buffer, rWidth * 3);
+      }
+      else
+      {
+	gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
+			   vnc->height - desc->header.r.y - rHeight,
+			   desc->header.r.x,
+			   rHeight, rWidth,
+			   GDK_RGB_DITHER_NORMAL, buffer, rHeight * 3);
       }
 
-      gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
-			 desc->header.r.x, desc->header.r.y,
-			 rWidth, rHeight,
-			 GDK_RGB_DITHER_NORMAL, buffer, rWidth * 3);
       g_free(buffer);
       break;
 
     case rfbEncodingCopyRect:
-      gdk_draw_pixmap(vnc->offscreen, vnc->gc, vnc->offscreen,
-		      desc->data.cr.srcX, desc->data.cr.srcY,
-		      desc->header.r.x, desc->header.r.y,
-		      rWidth, rHeight);
+      if (!vnc->m_bRotateDisplay)
+      {
+	gdk_draw_pixmap(vnc->offscreen, vnc->gc, vnc->offscreen,
+			desc->data.cr.srcX, desc->data.cr.srcY,
+			desc->header.r.x, desc->header.r.y,
+			rWidth, rHeight);
+      }
+      else
+      {
+	gdk_draw_pixmap(vnc->offscreen, vnc->gc, vnc->offscreen,
+			vnc->height - desc->data.cr.srcY - rHeight,
+			desc->data.cr.srcX,
+			vnc->height - desc->header.r.y - rHeight,
+			desc->header.r.x,
+			rHeight, rWidth);
+      }
       break;
 
     case rfbEncodingRRE:
+      //g_message("rre encoding");
       buffer = g_malloc(totPixels * 3);
       tmpR = (desc->data.rre.bgPixel >> fmt->redShift & fmt->redMax) *
 	redScale;
@@ -608,7 +735,15 @@ static void vnc_viewer_framebuffer_update(VncViewer *vnc,
 
       for (i = 0; i < totPixels; i++)
       {
-	buffer[3*i] = tmpR; buffer[3*i+1] = tmpG; buffer[3*i+2] = tmpB;
+	if (!vnc->m_bRotateDisplay)
+	{
+	  buffer[3*i] = tmpR; buffer[3*i+1] = tmpG; buffer[3*i+2] = tmpB;
+	}
+	else
+	{
+	  j = rWidth * ((i % rHeight) + 1) - ((i / rHeight) + 1);
+	  buffer[3*j] = tmpR; buffer[3*j+1] = tmpG; buffer[3*j+2] = tmpB;
+	}
       }
 
       for (srect = 0; srect < desc->data.rre.nSubrects; srect++)
@@ -618,25 +753,56 @@ static void vnc_viewer_framebuffer_update(VncViewer *vnc,
 	tmpR = (pixel >> fmt->redShift & fmt->redMax) * redScale;
 	tmpG = (pixel >> fmt->greenShift & fmt->greenMax) * greenScale;
 	tmpB = (pixel >> fmt->blueShift & fmt->blueMax) * blueScale;
-	for (j = sr->y; j < sr->y + sr->h; j++)
+
+	if (!vnc->m_bRotateDisplay)
 	{
-	  for (i = sr->x; i < sr->x + sr->w; i++)
+	  for (j = sr->y; j < sr->y + sr->h; j++)
 	  {
-	    buffer[3*(rWidth*j+i)]   = tmpR;
-	    buffer[3*(rWidth*j+i)+1] = tmpG;
-	    buffer[3*(rWidth*j+i)+2] = tmpB;
+	    for (i = sr->x; i < sr->x + sr->w; i++)
+	    {
+	      buffer[3*(rWidth*j+i)]   = tmpR;
+	      buffer[3*(rWidth*j+i)+1] = tmpG;
+	      buffer[3*(rWidth*j+i)+2] = tmpB;
+	    }
+	  }
+	}
+	else
+	{
+	  for (j = sr->y; j < sr->y + sr->h; j++)
+	  {
+	    for (i = sr->x; i < sr->x + sr->w; i++)
+	    {
+	      k = rHeight * (((rWidth * j + i) % rWidth) + 1) -
+		(((rWidth * j + i) / rWidth) + 1);
+	      buffer[3*k]   = tmpR;
+	      buffer[3*k+1] = tmpG;
+	      buffer[3*k+2] = tmpB;
+	    }
 	  }
 	}
       }
 
-      gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
-			 desc->header.r.x, desc->header.r.y,
-			 rWidth, rHeight,
-			 GDK_RGB_DITHER_NORMAL, buffer, rWidth * 3);
+      if (!vnc->m_bRotateDisplay)
+      {
+	gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
+			   desc->header.r.x, desc->header.r.y,
+			   rWidth, rHeight,
+			   GDK_RGB_DITHER_NORMAL, buffer, rWidth * 3);
+      }
+      else
+      {
+	gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
+			   vnc->height - desc->header.r.y - rHeight,
+			   desc->header.r.x,
+			   rHeight, rWidth,
+			   GDK_RGB_DITHER_NORMAL, buffer, rHeight * 3);
+      }
+
       g_free(buffer);
       break;
 
     case rfbEncodingCoRRE:
+      //g_message("corre encoding");
       buffer = g_malloc(totPixels * 3);
       tmpR = (desc->data.corre.bgPixel >> fmt->redShift & fmt->redMax) *
 	redScale;
@@ -647,7 +813,15 @@ static void vnc_viewer_framebuffer_update(VncViewer *vnc,
 
       for (i = 0; i < totPixels; i++)
       {
-	buffer[3*i] = tmpR; buffer[3*i+1] = tmpG; buffer[3*i+2] = tmpB;
+	if (!vnc->m_bRotateDisplay)
+	{
+	  buffer[3*i] = tmpR; buffer[3*i+1] = tmpG; buffer[3*i+2] = tmpB;
+	}
+	else
+	{
+	  j = rWidth * ((i % rHeight) + 1) - ((i / rHeight) + 1);
+	  buffer[3*j] = tmpR; buffer[3*j+1] = tmpG; buffer[3*j+2] = tmpB;
+	}
       }
 
       for (srect = 0; srect < desc->data.corre.nSubrects; srect++)
@@ -657,64 +831,138 @@ static void vnc_viewer_framebuffer_update(VncViewer *vnc,
 	tmpR = (pixel >> fmt->redShift & fmt->redMax) * redScale;
 	tmpG = (pixel >> fmt->greenShift & fmt->greenMax) * greenScale;
 	tmpB = (pixel >> fmt->blueShift & fmt->blueMax) * blueScale;
-	for (j = sr->y; j < sr->y + sr->h; j++)
+
+	if (!vnc->m_bRotateDisplay)
 	{
-	  for (i = sr->x; i < sr->x + sr->w; i++)
+	  for (j = sr->y; j < sr->y + sr->h; j++)
 	  {
-	    buffer[3*(rWidth*j+i)]   = tmpR;
-	    buffer[3*(rWidth*j+i)+1] = tmpG;
-	    buffer[3*(rWidth*j+i)+2] = tmpB;
+	    for (i = sr->x; i < sr->x + sr->w; i++)
+	    {
+	      buffer[3*(rWidth*j+i)]   = tmpR;
+	      buffer[3*(rWidth*j+i)+1] = tmpG;
+	      buffer[3*(rWidth*j+i)+2] = tmpB;
+	    }
+	  }
+	}
+	else
+	{
+	  for (j = sr->y; j < sr->y + sr->h; j++)
+	  {
+	    for (i = sr->x; i < sr->x + sr->w; i++)
+	    {
+	      k = rHeight * (((rWidth * j + i) % rWidth) + 1) -
+		(((rWidth * j + i) / rWidth) + 1);
+	      buffer[3*k]   = tmpR;
+	      buffer[3*k+1] = tmpG;
+	      buffer[3*k+2] = tmpB;
+	    }
 	  }
 	}
       }
 
-      gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
-			 desc->header.r.x, desc->header.r.y,
-			 rWidth, rHeight,
-			 GDK_RGB_DITHER_NORMAL, buffer, rWidth * 3);
+      if (!vnc->m_bRotateDisplay)
+      {
+	gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
+			   desc->header.r.x, desc->header.r.y,
+			   rWidth, rHeight,
+			   GDK_RGB_DITHER_NORMAL, buffer, rWidth * 3);
+      }
+      else
+      {
+	gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
+			   vnc->height - desc->header.r.y - rHeight,
+			   desc->header.r.x,
+			   rHeight, rWidth,
+			   GDK_RGB_DITHER_NORMAL, buffer, rHeight * 3);
+      }
+
       g_free(buffer);
       break;
 
+    // optimized hextile encoding only supports 8 and 16 bit colordepth
+    // Code depends on the documented GTK-structure GdkImage of the gpointer *mem
+    // May break in case this is changed in GDK
     case rfbEncodingHextile:
-      buffer = g_malloc(totPixels * 3);
-      bgR = bgG = bgB = 0;
+      //g_message("hextile encoding");
+      //Testing
+      if (bpp==8)
+      	buffer8=g_malloc(totPixels);
+      else
+      {
+	if (!vnc->m_bRotateDisplay)
+	  gdkimage=gdk_image_new(GDK_IMAGE_FASTEST, gdkvisual16, rWidth,
+				 rHeight);
+	else
+	  gdkimage=gdk_image_new(GDK_IMAGE_FASTEST, gdkvisual16, rHeight,
+				 rWidth);
+      }
+
       xPos = yPos = 0;
       for (tile = 0; tile < desc->data.hextile.nTiles; tile++)
       {
 	rfbHextile *ht = desc->data.hextile.tiles[tile];
-	
+
 	if (ht->type & rfbHextileRaw)
 	{
-	  for (j = 0; j < ht->height; j++)
+	  //g_message("got raw hextile");
+	  guint ii=0;
+	  if (bpp==8)
 	  {
-	    for (i = 0; i < ht->width; i++)
+	    if (!vnc->m_bRotateDisplay)
 	    {
-	      guint offs = (yPos + j) * rWidth + xPos + i;
-	      guint32 pixel = 0;
-
-	      switch (bpp)
+	      for (j = 0; j < ht->height; j++)
 	      {
-	      case 8:
-		pixel = desc->data.raw[i];
-		break;
-
-	      case 16:
-		pixel = ((guint16 *)(desc->data.raw))[i];
-		break;
-
-	      case 32:
-		pixel = ((guint32 *)(desc->data.raw))[i];
-		break;
-
-	      default:
-		g_assert_not_reached();
+		//for (i = 0; i < ht->width; i++)
+		// {
+		//	buffer8[(yPos+j)*rWidth+xPos+i]=(ht->data.raw)[ii++];
+		// }
+		memcpy(&buffer8[(yPos+j)*rWidth+xPos],&(ht->data.raw)[ii],ht->width);
+		ii+=ht->width;
 	      }
+	    }
+	    else
+	    {
+	      for (j = 0; j < ht->height; j++)
+	      {
+		for (i = 0; i < ht->width; i++)
+		{
+		  buffer8[((xPos + i) * rHeight) + (rHeight - yPos - j - 1)] =
+		    (ht->data.raw)[ii++];
+		}
+	      }
+	    }
+	  }
+	  else
+	  {
+	    if (!vnc->m_bRotateDisplay)
+	    {
+	      for (j = 0; j < ht->height; j++)
+	      {
+		memcpy(&((guint8 *) gdkimage->mem)[(yPos + j) * gdkimage->bpl + xPos * 2], //2 is gdkimage->bpp
+		       &(ht->data.raw)[ii], ht->width * 2);
 
-	      buffer[offs] = (pixel >> fmt->redShift & fmt->redMax) * redScale;
-	      buffer[offs+1] = (pixel >> fmt->greenShift & fmt->greenMax) * greenScale;
-	      buffer[offs+2] = (pixel >> fmt->blueShift & fmt->blueMax) * blueScale;
-	      if (offs == 0)
-		g_print("%06x ", pixel);
+		/*
+		for (i = 0; i < ht->width; i++)
+		{
+		  guint32 pixel = ((guint16 *)(ht->data.raw))[ii++];
+		  gdk_image_put_pixel(gdkimage, xPos + i, yPos + j, pixel);
+		}
+		*/
+		
+		ii += 2 * ht->width;
+	      }
+	    }
+	    else
+	    {
+	      for (j = 0; j < ht->height; j++)
+	      {
+		for (i = 0; i < ht->width; i++)
+		{
+		  guint32 pixel = ((guint16 *)(ht->data.raw))[ii++];
+		  gdk_image_put_pixel(gdkimage, rHeight - yPos - j - 1,
+				      xPos + i, pixel);
+		}
+	      }
 	    }
 	  }
 	}
@@ -722,20 +970,65 @@ static void vnc_viewer_framebuffer_update(VncViewer *vnc,
 	{
 	  if (ht->type & rfbHextileBackgroundSpecified)
 	  {
-	    bgR = (ht->data.rects.bgcolour >> fmt->redShift & fmt->redMax) * redScale;
-	    bgG = (ht->data.rects.bgcolour >> fmt->greenShift & fmt->greenMax) * greenScale;
-	    bgB = (ht->data.rects.bgcolour >> fmt->blueShift & fmt->blueMax) * blueScale;
-	    g_print("%06x ", ht->data.rects.bgcolour);
+	    bgRGB = ht->data.rects.bgcolour;
 	  }
 
-	  for (j = 0; j < ht->height; j++)
+	  if (bpp==8)
 	  {
-	    for (i = 0; i < ht->width; i++)
+	    if (!vnc->m_bRotateDisplay)
 	    {
-	      guint offs = (yPos + j) * rWidth + xPos + i;
-	      buffer[offs] = bgR;
-	      buffer[offs+1] = bgG;
-	      buffer[offs+2] = bgB;
+	      guint tmp=(yPos)*rWidth+xPos;
+	      for (j = 0; j < ht->height; j++)
+	      {
+		memset(&buffer8[tmp],bgRGB,ht->width);
+		tmp+=rWidth;
+	      }
+	    }
+	    else
+	    {
+	      guint tmp = (xPos * rHeight) + (rHeight - yPos - ht->height);
+	      for (j = 0; j < ht->width; j++)
+	      {
+		memset(&buffer8[tmp], bgRGB, ht->height);
+		tmp+=rHeight;
+	      }
+	    }
+	  }
+	  else
+	  {
+	    if (!vnc->m_bRotateDisplay)
+	    {
+	      guint16 * tmpimg = (guint16 *)
+		(&((guint8*)gdkimage->mem)[yPos * gdkimage->bpl + xPos*2]);
+	      for (j = 0; j < ht->height; j++)
+	      {
+		guint8 * tmp=(guint8*) tmpimg;
+		for (i = 0; i < ht->width; i++)
+		{
+		  //  gdk_image_put_pixel(gdkimage,xPos+i,yPos+j,bgRGB);
+		  *tmpimg=bgRGB;
+		  *tmpimg++;
+		}
+		tmpimg=(guint16*) (tmp+gdkimage->bpl);
+	      }
+	    }
+	    else
+	    {
+	      guint16 * tmpimg = (guint16 *)
+		(&((guint8*)gdkimage->mem)[xPos * gdkimage->bpl +
+					   ((rHeight - yPos - ht->height) *
+					    2)]);
+	      for (j = 0; j < ht->width; j++)
+	      {
+		guint8 * tmp = (guint8 *) tmpimg;
+		for (i = 0; i < ht->height; i++)
+		{
+		  //  gdk_image_put_pixel(gdkimage,xPos+i,yPos+j,bgRGB);
+		  *tmpimg=bgRGB;
+		  *tmpimg++;
+		}
+		tmpimg=(guint16*) (tmp+gdkimage->bpl);
+	      }
 	    }
 	  }
 
@@ -743,17 +1036,222 @@ static void vnc_viewer_framebuffer_update(VncViewer *vnc,
 	  {
 	    for (srect = 0; srect < ht->data.rects.nRects; srect++)
 	    {
-	      for (j = 0; j < ht->data.rects.rects[srect].h; j++)
-	      {
-		for (i = 0; i < ht->data.rects.rects[srect].w; i++)
-		{
-		  guint offs = (yPos + ht->data.rects.rects[srect].y + j) *
-		    rWidth + xPos + ht->data.rects.rects[srect].x + i;
-		  guint32 pixel = ht->data.rects.rects[srect].colour;
+	      guint16 pixel;
+	      guint8 wtype=0;
+	      if (ht->type & rfbHextileSubrectsColoured)
+		wtype=2;
+	      if (bpp==16)
+		wtype++;
 
-		  buffer[offs] = (pixel >> fmt->redShift & fmt->redMax) * redScale;
-		  buffer[offs+1] = (pixel >> fmt->greenShift & fmt->greenMax) * greenScale;
-		  buffer[offs+2] = (pixel >> fmt->blueShift & fmt->blueMax) * blueScale;
+	      if (!vnc->m_bRotateDisplay)
+	      {
+		switch (wtype)
+		{
+		case 0:
+		  // no subrect color, 8-bit pixel
+		  {
+		    pixel = ht->data.rects.fgcolour;
+		    guint tmp=
+		      (yPos +
+		       rfbHextileExtractY(ht->data.rects.rects0[srect].xy)) *
+		      rWidth + xPos +
+		      rfbHextileExtractX(ht->data.rects.rects0[srect].xy);
+		    for (j = 0; j < rfbHextileExtractH(ht->data.rects.rects0[srect].wh); j++)
+		    {
+		      memset(&buffer8[tmp], pixel,
+		           rfbHextileExtractW(ht->data.rects.rects0[srect].wh));
+		      tmp+=rWidth;
+		    }
+		  }
+		  break;
+		case 1:
+		  {
+		    //g_message("case1");
+		    // no subrect color, 16-bit pixel
+		    pixel = ht->data.rects.fgcolour;
+		    guint16 * tmpimg=(guint16*)
+		      (&((guint8*)gdkimage->mem)
+		       [(yPos+ rfbHextileExtractY(ht->data.rects.rects0[srect].xy))
+			*gdkimage->bpl
+			+(xPos+rfbHextileExtractX(ht->data.rects.rects0[srect].xy))*2]);
+		    for (j = 0; j < rfbHextileExtractH(ht->data.rects.rects0[srect].wh); j++)
+		    {
+		      //	if (yPos>750)
+		      //   g_message("j %d",j);
+		      guint8 * tmp=(guint8*) tmpimg;
+		      for (i = 0; i < rfbHextileExtractW(ht->data.rects.rects0[srect].wh); i++)
+		      {
+			//  gdk_image_put_pixel(gdkimage,xPos+i,yPos+j,pixel);
+			*tmpimg=pixel;
+			*tmpimg++;
+		      }
+		      tmpimg=(guint16*) (tmp+gdkimage->bpl);
+		      // guint tmp=yPos + rfbHextileExtractY(ht->data.rects.rects0[srect].xy)+j;
+		      // guint tmp2=xPos + rfbHextileExtractX(ht->data.rects.rects0[srect].xy);
+		      // for (i = 0; i < rfbHextileExtractW(ht->data.rects.rects0[srect].wh); i++)
+		      // {
+		      //   gdk_image_put_pixel(gdkimage, tmp2++, tmp, pixel);
+		      // }
+		    }
+		  }
+		  break;
+		case 2:
+		  // subrect color, 8-bit pixel
+		  {
+		    pixel=ht->data.rects.rects1[srect].colour;
+		    guint tmp=(yPos+ rfbHextileExtractY(ht->data.rects.rects1[srect].xy))*rWidth
+		      +xPos+ rfbHextileExtractX(ht->data.rects.rects1[srect].xy);
+		    for (j = 0; j < rfbHextileExtractH(ht->data.rects.rects1[srect].wh); j++)
+		    {
+		      memset(&buffer8[tmp], 
+			     pixel,rfbHextileExtractW(ht->data.rects.rects1[srect].wh));
+		      tmp+=rWidth;
+		    }
+		  }
+		  break;
+		case 3:
+		  {
+		    //g_message("case3");
+		    // subrect color, 16-bit pixel
+		    pixel=ht->data.rects.rects2[srect].colour;
+		    guint16 * tmpimg=(guint16*)
+		      (&((guint8*)gdkimage->mem)
+		       [(yPos+ rfbHextileExtractY(ht->data.rects.rects2[srect].xy))
+			*gdkimage->bpl
+			+(xPos+rfbHextileExtractX(ht->data.rects.rects2[srect].xy))*2]);
+		    for (j = 0; j < rfbHextileExtractH(ht->data.rects.rects2[srect].wh); j++)
+		    {
+		      // if (yPos>750)
+		      //   g_message("yPos %d Y %d j %d",yPos,j);
+		      guint8 * tmp=(guint8*) tmpimg;
+		      for (i = 0; i < rfbHextileExtractW(ht->data.rects.rects2[srect].wh); i++)
+		      {
+			//  gdk_image_put_pixel(gdkimage,xPos+i,yPos+j,pixel);
+			*tmpimg=pixel;
+			*tmpimg++;
+		      }
+		      tmpimg=(guint16*) (tmp+gdkimage->bpl);
+		      // guint tmp=yPos + rfbHextileExtractY(ht->data.rects.rects0[srect].xy)+j;
+		      // guint tmp2=xPos + rfbHextileExtractX(ht->data.rects.rects0[srect].xy);
+		      // for (i = 0; i < rfbHextileExtractW(ht->data.rects.rects0[srect].wh); i++)
+		      // {
+		      //   gdk_image_put_pixel(gdkimage, tmp2++, tmp, pixel);
+		      // }
+		    }
+		  }
+		  break;
+		}
+	      }
+	      else
+	      {
+		// rotated display
+		switch (wtype)
+		{
+		case 0:
+		  // no subrect color, 8-bit pixel
+		  {
+		    pixel = ht->data.rects.fgcolour;
+
+		    guint tmp =
+		      ((xPos +
+			rfbHextileExtractX(ht->data.rects.rects0[srect].xy)) *
+		       rHeight) +
+		      (rHeight - yPos -
+		       rfbHextileExtractH(ht->data.rects.rects0[srect].wh) -
+		       rfbHextileExtractY(ht->data.rects.rects0[srect].xy));
+
+		    for (j = 0; j < rfbHextileExtractW(ht->data.rects.rects0[srect].wh); j++)
+		    {
+		      memset(&buffer8[tmp], pixel,
+			     rfbHextileExtractH(ht->data.rects.rects0[srect].wh));
+		      tmp+=rHeight;
+		    }
+		  }
+		  break;
+		case 1:
+		  {
+		    //g_message("case1");
+		    // no subrect color, 16-bit pixel
+		    pixel = ht->data.rects.fgcolour;
+
+		    guint16 *tmpimg = (guint16 *)
+		      (&((guint8 *) gdkimage->mem)
+		      [((xPos +
+			 rfbHextileExtractX(ht->data.rects.rects0[srect].xy)) *
+			gdkimage->bpl) +
+		       ((rHeight - yPos -
+			 rfbHextileExtractH(ht->data.rects.rects0[srect].wh) -
+			 rfbHextileExtractY(ht->data.rects.rects0[srect].xy)) *
+			 2)]);
+
+		    for (j = 0; j < rfbHextileExtractW(ht->data.rects.rects0[srect].wh); j++)
+		    {
+		      //	if (yPos>750)
+		      //   g_message("j %d",j);
+		      guint8 *tmp = (guint8 *) tmpimg;
+		      for (i = 0; i < rfbHextileExtractH(ht->data.rects.rects0[srect].wh); i++)
+		      {
+			*tmpimg=pixel;
+			*tmpimg++;
+		      }
+
+		      tmpimg = (guint16 *) (tmp + gdkimage->bpl);
+		    }
+		  }
+		  break;
+		case 2:
+		  // subrect color, 8-bit pixel
+		  {
+		    pixel=ht->data.rects.rects1[srect].colour;
+
+		    guint tmp =
+		      ((xPos +
+			rfbHextileExtractX(ht->data.rects.rects1[srect].xy)) *
+		       rHeight) +
+		      (rHeight - yPos -
+		       rfbHextileExtractH(ht->data.rects.rects1[srect].wh) -
+		       rfbHextileExtractY(ht->data.rects.rects1[srect].xy));
+
+		    for (j = 0; j < rfbHextileExtractW(ht->data.rects.rects1[srect].wh); j++)
+		    {
+		      memset(&buffer8[tmp], 
+			     pixel,rfbHextileExtractH(ht->data.rects.rects1[srect].wh));
+		      tmp+=rHeight;
+		    }
+		  }
+		  break;
+		case 3:
+		  {
+		    //g_message("case3");
+		    // subrect color, 16-bit pixel
+		    pixel=ht->data.rects.rects2[srect].colour;
+
+		    guint16 *tmpimg = (guint16 *)
+		      (&((guint8 *) gdkimage->mem)
+		      [((xPos +
+			 rfbHextileExtractX(ht->data.rects.rects2[srect].xy)) *
+			gdkimage->bpl) +
+		       ((rHeight - yPos -
+			 rfbHextileExtractH(ht->data.rects.rects2[srect].wh) -
+			 rfbHextileExtractY(ht->data.rects.rects2[srect].xy)) *
+			 2)]);
+
+		    //g_message("initial tmpimg values are: xPos=%u, x2=%u, rHeight=%u, yPos=%u, h2=%u, y2=%u, w2=%u, rWidth=%u, w1=%u, h1=%u, srect=%u, ht=%p.\n", xPos, rfbHextileExtractX(ht->data.rects.rects2[srect].xy), rHeight, yPos, rfbHextileExtractH(ht->data.rects.rects2[srect].wh), rfbHextileExtractY(ht->data.rects.rects2[srect].xy), rfbHextileExtractW(ht->data.rects.rects2[srect].wh), rWidth, ht->width, ht->height, srect, ht);
+
+		    for (j = 0; j < rfbHextileExtractW(ht->data.rects.rects2[srect].wh); j++)
+		    {
+		      // if (yPos>750)
+		      //   g_message("yPos %d Y %d j %d",yPos,j);
+		      guint8 *tmp = (guint8 *) tmpimg;
+		      for (i = 0; i < rfbHextileExtractH(ht->data.rects.rects2[srect].wh); i++)
+		      {
+			*tmpimg = pixel;
+			*tmpimg++;
+		      }
+		      tmpimg=(guint16*) (tmp+gdkimage->bpl);
+		    }
+		  }
+		  break;
 		}
 	      }
 	    }
@@ -766,29 +1264,121 @@ static void vnc_viewer_framebuffer_update(VncViewer *vnc,
 	  xPos = 0;
 	  yPos += ht->height;
 	}
+	// g_message("xpos %d ypos %d",xPos,yPos);
       }
 
-      gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
+      // gdkimage in the case of hextile 16 bit encoding, buffer8 in case of hextile 8 bit, 
+      // buffer in case of no hextile encoding
+      if (!gdkimage)
+      {
+	if (!buffer8)
+	{
+	  if (!vnc->m_bRotateDisplay)
+	  {
+	    gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
+			       desc->header.r.x, desc->header.r.y,
+			       rWidth, rHeight,
+			       GDK_RGB_DITHER_NORMAL, buffer, rWidth * 3);
+	  }
+	  else
+	  {
+	    gdk_draw_rgb_image(vnc->offscreen, vnc->gc,
+			       vnc->height - desc->header.r.y - rHeight,
+			       desc->header.r.x,
+			       rHeight, rWidth,
+			       GDK_RGB_DITHER_NORMAL, buffer, rHeight * 3);
+	  }
+	  g_free(buffer);
+	}
+	else
+	{
+	  //g_message("draw attempt");
+	  if (!vnc->m_bRotateDisplay)
+	  {
+	    gdk_draw_indexed_image(vnc->offscreen, vnc->gc,
+				   desc->header.r.x, desc->header.r.y,
+				   rWidth, rHeight,
+				   GDK_RGB_DITHER_NONE, buffer8, rWidth, cmap);
+	  }
+	  else
+	  {
+	    gdk_draw_indexed_image(vnc->offscreen, vnc->gc,
+	    			   vnc->height - desc->header.r.y - rHeight,
+				   desc->header.r.x,
+				   rHeight, rWidth,
+				   GDK_RGB_DITHER_NONE, buffer8, rHeight, cmap);
+	  }
+	  g_free(buffer8);
+	  buffer8=NULL;
+	  //g_message("draw done");
+	}
+      }
+      else
+      {
+	if (!vnc->m_bRotateDisplay)
+	{
+	  gdk_draw_image(vnc->offscreen, vnc->gc, gdkimage, 0, 0,
 			 desc->header.r.x, desc->header.r.y,
-			 rWidth, rHeight,
-			 GDK_RGB_DITHER_NORMAL, buffer, rWidth * 3);
-      g_free(buffer);
+			 rWidth, rHeight);
+	}
+	else
+	{
+	  gdk_draw_image(vnc->offscreen, vnc->gc, gdkimage, 0, 0,
+			 vnc->height - desc->header.r.y - rHeight,
+			 desc->header.r.x,
+			 rHeight, rWidth);
+	}
+	g_object_unref(gdkimage);
+      }
       break;
 
     default:
       g_warning("Unknown rectangle encoding returned");
     }
 
-    gdk_draw_pixmap(GTK_WIDGET(vnc)->window,
-		    vnc->gc,
-		    vnc->offscreen,
-		    desc->header.r.x, desc->header.r.y,
-		    desc->header.r.x,
-		    desc->header.r.y,
-		    rWidth, rHeight);
+    // g_message("drawpix");
+    // draw the drawable to the window (I guess)
+    if (!vnc->m_bRotateDisplay)
+    {
+      gdk_draw_pixmap(GTK_WIDGET(vnc)->window,
+		      vnc->gc,
+		      vnc->offscreen,
+		      desc->header.r.x, desc->header.r.y,
+		      desc->header.r.x,
+		      desc->header.r.y,
+		      rWidth, rHeight);
+    }
+    else
+    {
+      gdk_draw_pixmap(GTK_WIDGET(vnc)->window,
+		      vnc->gc,
+		      vnc->offscreen,
+		      vnc->height - desc->header.r.y - rHeight,
+		      desc->header.r.x,
+		      vnc->height - desc->header.r.y - rHeight,
+		      desc->header.r.x,
+		      rHeight, rWidth);
+    }
+    //g_message("freeRect");
     freeRectangleDescription(desc);
+    // g_free(desc); 
+#ifdef perf
+    T2+=g_timer_elapsed(Tperf,NULL);
+#endif
+    // g_timer_destroy(T);
   }
 
+#ifdef perf
+  frames++;
+  if (ReadBytes>1000000)
+  {
+    Tcount=0;
+    T3+=g_timer_elapsed(Tperf,NULL);
+    g_message("%ld time T1 %f T2-T1 %f %f %f frames %d time measured %f Datarate %f",ReadBytes,
+	      T1,T2-T1,T1*1000000/ReadBytes,(T2-T1)*1000000/ReadBytes,frames,T3,ReadBytes/T3);
+    T1=0; T2=0; T3=0;ReadBytes=0; frames=0;
+  }
+#endif
   gtk_timeout_add(25, (GtkFunction) vnc_viewer_send_draw_request, vnc);
 }
 
@@ -796,7 +1386,17 @@ static void vnc_viewer_set_colourmap_entries(VncViewer *vnc,
 					     rfbServerToClientMsg *msg,
 					     guint16 *entries)
 {
-}
+  g_message("colormap called %d %d",msg->scme.firstColour,msg->scme.nColours);
+  int n=msg->scme.firstColour;
+  int j;
+  for (j=0;j<msg->scme.nColours;j++)
+  {
+    cmap->colors[n+j]=(((guint32)entries[3*j]&0xFF00)<<8) | 
+      ((guint32)entries[3*j+1]&0xFF00) |
+      ((guint32)entries[3*j+2]>>8);
+    //g_message("c %d - %lX",n+j,cmap->colors[n+j]);
+  }
+}	
 
 static void vnc_viewer_bell(VncViewer *vnc, rfbServerToClientMsg *msg)
 {
@@ -853,10 +1453,22 @@ static void vnc_viewer_realize(GtkWidget *widget)
 
   if (vnc->offscreen)
     gdk_pixmap_unref(vnc->offscreen);
-  vnc->offscreen = gdk_pixmap_new(widget->window,
-				  vnc->width,
-				  vnc->height,
-				  -1);
+  if (!vnc->m_bRotateDisplay)
+  {
+    vnc->offscreen = gdk_pixmap_new(widget->window,
+				    vnc->width,
+				    vnc->height,
+				    -1);
+  }
+  else
+  {
+    vnc->offscreen = gdk_pixmap_new(widget->window,
+				    vnc->height,
+				    vnc->width,
+				    -1);
+  }
+  g_message("drawable depth %d",gdk_drawable_get_depth(widget->window));
+
   if (vnc->gc)
     gdk_gc_unref(vnc->gc);
   vnc->gc = gdk_gc_new(widget->window);
@@ -901,13 +1513,26 @@ static gint vnc_viewer_expose_event(GtkWidget *widget, GdkEventExpose *event)
   if (GTK_WIDGET_DRAWABLE(widget))
   {
     // gdk_draw_pixmap is deprecated, should be replaced
-    gdk_draw_pixmap(widget->window,
-		    vnc->gc,
-		    vnc->offscreen,
-		    event->area.x,
-		    event->area.y,
-		    event->area.x,     event->area.y,
-		    event->area.width, event->area.height);
+    if (!vnc->m_bRotateDisplay)
+    {
+      gdk_draw_pixmap(widget->window,
+		      vnc->gc,
+		      vnc->offscreen,
+		      event->area.x,
+		      event->area.y,
+		      event->area.x,     event->area.y,
+		      event->area.width, event->area.height);
+    }
+    else
+    {
+      gdk_draw_pixmap(widget->window,
+		      vnc->gc,
+		      vnc->offscreen,
+		      event->area.x,
+		      event->area.y,
+		      event->area.x,     event->area.y,
+		      event->area.width, event->area.height);
+    }
   }
 
   return FALSE;
@@ -940,10 +1565,17 @@ static gint vnc_viewer_button_press_event(GtkWidget *widget,
     break;
   }
 
-  x = (guint16) event->x;
-  y = (guint16) event->y;
+  if (!vnc->m_bRotateDisplay)
+  {
+    x = (guint16) event->x;
+    y = (guint16) event->y;
+  }
+  else
+  {
+    x = (guint16) event->y;
+    y = (guint16) vnc->height - event->x;
+  }
   vnc_viewer_queue_event(vnc, newPointerEventMsg(vnc->button_mask, x, y));
-					    
   return TRUE;
 }
 
@@ -974,10 +1606,17 @@ static gint vnc_viewer_button_release_event(GtkWidget *widget,
     break;
   }
 
-  x = (guint16) event->x;
-  y = (guint16) event->y;
+  if (!vnc->m_bRotateDisplay)
+  {
+    x = (guint16) event->x;
+    y = (guint16) event->y;
+  }
+  else
+  {
+    x = (guint16) event->y;
+    y = (guint16) vnc->height - event->x;
+  }
   vnc_viewer_queue_event(vnc, newPointerEventMsg(vnc->button_mask, x, y));
-
   return TRUE;
 }
 
@@ -989,8 +1628,16 @@ static gint vnc_viewer_motion_notify_event(GtkWidget *widget,
   if (vnc->fd < 0)
     return FALSE;
 
-  vnc->last_pointer_x_loc = (guint16) event->x;
-  vnc->last_pointer_y_loc = (guint16) event->y;
+  if (!vnc->m_bRotateDisplay)
+  {
+    vnc->last_pointer_x_loc = (guint16) event->x;
+    vnc->last_pointer_y_loc = (guint16) event->y;
+  }
+  else
+  {
+    vnc->last_pointer_x_loc = (guint16) event->y;
+    vnc->last_pointer_y_loc = (guint16) vnc->height - event->x;
+  }
   vnc_viewer_queue_event(vnc, newPointerEventMsg(vnc->button_mask,
 						 vnc->last_pointer_x_loc,
 						 vnc->last_pointer_y_loc));
@@ -1061,7 +1708,33 @@ gint vnc_viewer_key_press_event(GtkWidget *widget,
     }
   }
 
-  vnc_viewer_queue_event(vnc, newKeyEventMsg(TRUE, event->keyval));
+  guint32 keyval = event->keyval;
+  if (vnc->m_bRotateDisplay)
+  {
+    switch (keyval)
+    {
+    case GDK_Up:
+      keyval = GDK_Left;
+      break;
+
+    case GDK_Down:
+      keyval = GDK_Right;
+      break;
+
+    case GDK_Left:
+      keyval = GDK_Down;
+      break;
+
+    case GDK_Right:
+      keyval = GDK_Up;
+      break;
+
+    default:
+      break;
+    }
+  }
+
+  vnc_viewer_queue_event(vnc, newKeyEventMsg(TRUE, keyval));
 
   return TRUE;
 }
@@ -1156,7 +1829,33 @@ gint vnc_viewer_key_release_event(GtkWidget *widget,
       gtk_im_context_filter_keypress(vnc->im_context, event))
     return TRUE;
 
-  vnc_viewer_queue_event(vnc, newKeyEventMsg(FALSE, event->keyval));
+  guint32 keyval = event->keyval;
+  if (vnc->m_bRotateDisplay)
+  {
+    switch (keyval)
+    {
+    case GDK_Up:
+      keyval = GDK_Left;
+      break;
+
+    case GDK_Down:
+      keyval = GDK_Right;
+      break;
+
+    case GDK_Left:
+      keyval = GDK_Down;
+      break;
+
+    case GDK_Right:
+      keyval = GDK_Up;
+      break;
+
+    default:
+      break;
+    }
+  }
+
+  vnc_viewer_queue_event(vnc, newKeyEventMsg(FALSE, keyval));
 
   //if (GTK_WIDGET_CLASS(parent_class)->key_press_event(widget, event))
   //  return TRUE;
